/* FILE:
      GloveDriver.cpp

   VERSION:
      1.0

   PURPOSE:
      Windows 95/NT driver for the 5DT's Fifth Glove.

   COMMENTS:
      THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY OF ANY
      KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE
      IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A PARTICULAR
      PURPOSE.

   Copyright (c) 1997 Fifth Dimension Technologies. All Rights Reserved.
*/

#include "windows.h"

#define DllExport __declspec( dllexport )
#define RETRIES 3 // number of glove initialisation attempts

HANDLE		idComDev;
BOOL		m_fConnected;
BYTE		abIn[16];
DWORD       dwLength;

DWORD FAR PASCAL CommWatchProc(LPSTR lpData)
{
	while (m_fConnected) {
		WriteFile(idComDev, "C", 1, &dwLength, NULL); // set glove to "report data mode"
			BYTE header;
			do
				ReadFile( idComDev, &header, 1, &dwLength, NULL);
			while (header != 0x80);
			ReadFile( idComDev, abIn, 8, &dwLength, NULL);
			unsigned char checksum = 0;	// calculate checksum
			for (int i = 0; i < 8; i++)
				checksum ^= abIn[i];
			if (checksum != 0)
				OutputDebugString("GloveDriver: Glove checksum error\n");
	}
	return TRUE;
}

DllExport BOOL FAR PASCAL GloveInit(LPCSTR lpszPort)
{
	idComDev = 0;
	m_fConnected = FALSE;
	
	// open COMM device
	if ((idComDev = CreateFile(
		lpszPort, // communications port
		GENERIC_READ | GENERIC_WRITE, // open port for two-way communication
		0, // exclusive access
		NULL, // no security attrs
		OPEN_EXISTING,	// comm devices must use OPEN_EXISTING
		0, // not overlapped I/O
		NULL
		)) == INVALID_HANDLE_VALUE) {
		return FALSE; // create file failed - return FALSE
	}
	// setup device buffers
	SetupComm(idComDev, 256, 256);
	// set up timeouts
	COMMTIMEOUTS CommTimeOuts;
	CommTimeOuts.ReadIntervalTimeout = 1000;
	CommTimeOuts.ReadTotalTimeoutMultiplier = 1;
	CommTimeOuts.ReadTotalTimeoutConstant = 200;
	CommTimeOuts.WriteTotalTimeoutMultiplier = 1;
	CommTimeOuts.WriteTotalTimeoutConstant = 200;
	SetCommTimeouts(idComDev, &CommTimeOuts);
	DCB dcb;
	dcb.DCBlength = sizeof(DCB);
	GetCommState(idComDev, &dcb);
	// communication properties
	dcb.BaudRate = CBR_19200;
	dcb.ByteSize = 8;
	dcb.Parity = NOPARITY;
	dcb.StopBits = ONESTOPBIT;
	// setup hardware flow control
	dcb.fOutxDsrFlow = FALSE;
	dcb.fDtrControl = DTR_CONTROL_ENABLE;
	dcb.fOutxCtsFlow = FALSE;
	dcb.fRtsControl = RTS_CONTROL_ENABLE;
	// setup software flow control
	dcb.fInX = dcb.fOutX = FALSE;
	// other settings
	dcb.fBinary = TRUE;
	dcb.fParity = TRUE;
	SetCommState(idComDev, &dcb);
	for(int retry = 0; retry < RETRIES; retry++) { // initialise glove
		m_fConnected = TRUE;
		WriteFile(idComDev, "A", 1, &dwLength, NULL); // issue reset command
		Sleep(100);
		ReadFile( idComDev, abIn, 1, &dwLength, NULL);
		if (abIn[0] == 'U') // glove returns 'U' = OK
			break;
		Sleep(100);
	}
	if (retry == RETRIES) {
		CloseHandle(idComDev);
		return FALSE; // failed to initialise glove - return FALSE
	}
	// Create a secondary thread to monitor communications
	DWORD dwThreadID;
	if (NULL == 
		CreateThread((LPSECURITY_ATTRIBUTES) NULL,
		0, 
		(LPTHREAD_START_ROUTINE) CommWatchProc,
		NULL,
		0,
		&dwThreadID))
	{
		CloseHandle(idComDev);
		return FALSE;
	}
	return TRUE;
}

DllExport LPBYTE FAR PASCAL GloveGetDataPtr(void)
{
	return abIn;
}

DllExport BYTE FAR PASCAL GloveGetData(int nIndex)
{
	if (nIndex >= 0 && nIndex <= 6)
		return abIn[nIndex];
	return 0;
}
